import { Directive, HostBinding, Input, NgModule } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';
import { __spread } from 'tslib';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var isJsons = function (array) {
    return Array.isArray(array) &&
        array.every(function (row) { return typeof row === 'object' && !(row instanceof Array); });
};
/** @type {?} */
var isArrays = function (array) {
    return Array.isArray(array) && array.every(function (row) { return Array.isArray(row); });
};
/**
 * @param {?} array
 * @return {?}
 */
function jsonsHeaders(array) {
    return Array.from(new Set(array.map(function (item) { return Object.keys(item); }).reduce(function (a, b) { return __spread(a, b); }, [])));
}
/**
 * @param {?} jsons
 * @param {?=} headers
 * @return {?}
 */
function jsons2arrays(jsons, headers) {
    headers = headers || jsonsHeaders(jsons);
    // allow headers to have custom labels, defaulting to having the header data key be the label
    /** @type {?} */
    var headerLabels = (/** @type {?} */ (headers));
    /** @type {?} */
    var headerKeys = (/** @type {?} */ (headers));
    if (isJsons(headers)) {
        headerLabels = ((/** @type {?} */ (headers))).map(function (header) { return header.label; });
        headerKeys = ((/** @type {?} */ (headers))).map(function (header) { return header.key; });
    }
    /** @type {?} */
    var data = jsons.map(function (object) {
        return headerKeys.map(function (header) { return (header in object ? object[header] : ''); });
    });
    return __spread([headerLabels], data);
}
/** @type {?} */
var elementOrEmpty = function (element) {
    return element || element === 0 ? element : '';
};
/**
 * @param {?} data
 * @param {?=} delimiter
 * @return {?}
 */
function joiner(data, delimiter) {
    if (delimiter === void 0) { delimiter = ','; }
    return data
        .map(function (row, index) {
        return row.map(function (element) { return '"' + elementOrEmpty(element) + '"'; }).join(delimiter);
    })
        .join("\n");
}
/**
 * @param {?} data
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
function arrays2csv(data, headers, delimiter) {
    return joiner(headers ? __spread([headers], data) : data, delimiter);
}
/**
 * @param {?} data
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
function jsons2csv(data, headers, delimiter) {
    return joiner(jsons2arrays(data, headers), delimiter);
}
/**
 * @param {?} data
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
function string2csv(data, headers, delimiter) {
    return headers ? headers.join(delimiter) + "\n" + data : data;
}
/**
 * @param {?} data
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
function toCSV(data, headers, delimiter) {
    if (isJsons((/** @type {?} */ (data)))) {
        return jsons2csv((/** @type {?} */ (data)), headers, delimiter);
    }
    if (isArrays((/** @type {?} */ (data)))) {
        return arrays2csv((/** @type {?} */ (data)), headers, delimiter);
    }
    if (typeof data === 'string') {
        return string2csv(data, (/** @type {?} */ (headers)), delimiter);
    }
    throw new TypeError("Data should be a \"String\", \"Array of arrays\" OR \"Array of objects\" ");
}
/**
 * @param {?} data
 * @param {?=} uFEFF
 * @param {?=} headers
 * @param {?=} delimiter
 * @return {?}
 */
function buildURI(data, uFEFF, headers, delimiter) {
    if (uFEFF === void 0) { uFEFF = true; }
    /** @type {?} */
    var csv = toCSV(data, headers, delimiter);
    /** @type {?} */
    var blob = new Blob([uFEFF ? '\uFEFF' : '', csv], { type: 'text/csv' });
    return URL.createObjectURL(blob);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var CsvDirective = /** @class */ (function () {
    function CsvDirective(sanitizer) {
        this.sanitizer = sanitizer;
        /**
         * the body of the csv
         */
        this.data = [];
        /**
         * Set the seperator between values
         */
        this.delimiter = ',';
        /**
         * adds a Byte order mark to setup the csv as UTF-8
         */
        this.uFEFF = true;
        /**
         * filename
         */
        this.download = 'data.csv';
        this.target = '_blank';
    }
    Object.defineProperty(CsvDirective.prototype, "filename", {
        /** Set the filename of the csv. Default is `data.csv` */
        set: /**
         * Set the filename of the csv. Default is `data.csv`
         * @param {?} a
         * @return {?}
         */
        function (a) {
            this.download = a;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    CsvDirective.prototype.ngOnChanges = /**
     * @return {?}
     */
    function () {
        this.href = this.sanitizer.bypassSecurityTrustResourceUrl(buildURI(this.data, this.uFEFF, this.headers, this.delimiter));
    };
    CsvDirective.decorators = [
        { type: Directive, args: [{ selector: 'a[csvLink]' },] }
    ];
    /** @nocollapse */
    CsvDirective.ctorParameters = function () { return [
        { type: DomSanitizer }
    ]; };
    CsvDirective.propDecorators = {
        data: [{ type: Input }],
        headers: [{ type: Input }],
        delimiter: [{ type: Input }],
        filename: [{ type: Input }],
        uFEFF: [{ type: Input }],
        href: [{ type: HostBinding }],
        download: [{ type: HostBinding }],
        target: [{ type: Input }, { type: HostBinding }]
    };
    return CsvDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var CsvModule = /** @class */ (function () {
    function CsvModule() {
    }
    CsvModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [CsvDirective],
                    exports: [CsvDirective],
                },] }
    ];
    return CsvModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { CsvModule, CsvDirective };

//# sourceMappingURL=ctrl-ngx-csv.js.map